import oscP5.*;
import netP5.*;


OscP5 oscP5;
NetAddress myRemoteLocation;


int nextOut = 0;
int sourceGrabbed = -1;
int numSources = 1;
int maxSources = 16;
int numPresets = 11;
int numMovers = 5;
float pixScale = 1.0 / 70; // scaling from pixel values to meters ( 90 pixels / 45 = 2 meters)
int originY = 800; // amount line is displaced from top
int originX = 300; // amount line is displaced from left
SSource[] mySources;
button bMakeSource, bRemoveSource, bChangeSource,bRecord,bPlayback;
button[] preset;


void setup() {
  
  size(600,800);
  frameRate(40);
  oscP5 = new OscP5(this,7000);
  myRemoteLocation = new NetAddress("127.0.0.1",7000);

  bMakeSource = new button(10,10,25,25,color(120));
  bRemoveSource = new button(40,10,25,25,color(80));
  bChangeSource = new button(360,10,25,25,color(160));
  bRecord = new button(80,10,50,25,color(100));
  bPlayback = new button(80,10,50,25,color(100));

  preset = new button[numPresets];
  mySources = new SSource[maxSources];

  for (int i=0;i<maxSources;i++){
    mySources[i] = new SSource(999999, 9999999, 0, 1.0, i);
  }
  for (int i=0;i<6;i++){
    preset[i] = new button(120 + i * 20,10,15,15,color(100));
  }
  for (int i=6;i<numPresets;i++){
    preset[i] = new button(560, 10 + i * 20,15,15,color(100));
  }
}


class Movement {
  Movement(){
  }
  void doMove(SSource source){
  }
}

class CircleMove extends Movement{
  CircleMove(){
  }
  void doMove(SSource source){
    source.phase1 += .01;
    source.move(int(310 + 160 * sin(source.phase1)), int(200 * cos(source.phase1)) + 560);
    if (source.phase1 > TWO_PI) source.phase1 -= TWO_PI;
  }
}

class EightMove extends Movement {
  EightMove(){
  }
  void doMove(SSource source){
    source.phase1 += 0.041;
    source.phase2 += 0.023;
    source.move(int(300 + 180 * sin(source.phase1)), int(180 * cos(source.phase2)) + 500);
    if (source.phase1 > TWO_PI) source.phase1 -= TWO_PI;
    if (source.phase2 > TWO_PI) source.phase2 -= TWO_PI;
  }
}

class FarMove extends Movement {
  FarMove(){
  }
  void doMove(SSource source){
    source.phase1 += 0.1;
    source.move(260, int(abs(94 * source.phase1) + 120));
    if (source.phase1 > TWO_PI) source.phase1 -= 2 * TWO_PI;

  }
}

class AcrossMove extends Movement {
  AcrossMove(){
  }
  void doMove(SSource source){
    source.phase1 += 0.03;
    source.move(int(80 * abs(source.phase1) + 80), 500);
    if (source.phase1 > TWO_PI) source.phase1 -= 2 * TWO_PI;

  }
}
class OutMove extends Movement {
  OutMove(){
  }
  void doMove(SSource source){
    source.phase1 -= 20;
    source.move(int(source.phase2), int(source.phase1));
  }
}

class button{
  int xStart,yStart,xSize,ySize;
  boolean mPressed;
  color mColor, mDefault;

  button(int x, int y, int w, int h, color c){
    xStart = x; 
    yStart = y; 
    xSize = w; 
    ySize = h; 
    mPressed = false; 
    mColor = c;
    mDefault = c;
  }
  boolean hovering(){
    if (mouseX > xStart && mouseX < xStart + xSize && mouseY > yStart && mouseY < yStart + ySize){
      return true;
    }
    else{
      return false;
    }
  }
  void drawObj(){
    fill(mColor);
    rect(xStart, yStart, xSize, ySize);
  }

  void pressing(){
    if (mPressed == false){
      mPressed = true;
      mColor = color(120,200,200);
    }
  }
  void releasing(){
    if (mPressed == true){
      mPressed = false;
      mColor = mDefault;
    }
  }

}

class SSource
{
  int xPos,yPos;
  int sourceNum;
  int sourceType;
  float sourceGain;
  boolean alive;
  color[] mColor;

  Movement movement;
  float phase1, phase2;

  SSource(int x, int y, int type, float gain, int num){
    xPos = x;
    yPos = y;
    sourceType = type;
    sourceGain = gain;
    alive = false;
    sourceNum = num;
    colorMode(HSB);
    mColor = new color[2];
    mColor[0] = color((sourceNum) * 15 + 10, 250 - sourceNum * 10, 250);
    mColor[1] = color((sourceNum) * 15 + 10, 250 - sourceNum * 10, 50 + sourceNum * 10);
    colorMode(RGB);
  }
  void move(int x,int y){
    xPos = x;
    yPos = y;
    sendMovement(this);
  }
  void changeType(int type){
    sourceType = type;
    sendMovement(this);
  } 
  void goAway(){
    alive = false;
    movement = null;
    sourceGain = 0.0;
    move(xPos, yPos);
  }

  // change to a position instead
  void comeBack(int x, int y){
    alive = true;
    sourceType = 0;
    sourceGain = 1.0;
    move(x ,y);
    phase1 = 0.0; 
    phase2 = 0.0;
  }
  void drawObj(){
    fill(mColor[sourceType]);
    rect(xPos-5,yPos-5,10,10);
    if (sourceType == 1){
      strokeWeight(1);
      stroke(204, 102, 0);
      line(xPos,yPos,originX, originY);
    }
  }

}

void draw() {
  background(0);  
  smooth();
  strokeWeight(4);
  stroke(204, 102, 0);
  fill(0);
  //  ellipse(201,201,90,90);
  line(-1.84 / pixScale + originX, originY - 1.0 / pixScale, 1.84 / pixScale + originX, originY - 1.0 / pixScale);
  bMakeSource.drawObj();
  bRemoveSource.drawObj();
  bChangeSource.drawObj();
  for(int i=0;i<numPresets;i++){
    preset[i].drawObj();
  }
  noStroke();

  for(int i=0;i<maxSources;i++){
    if (mySources[i].alive == true){
      if (mySources[i].movement != null){
        mySources[i].movement.doMove(mySources[i]);
      }
      mySources[i].drawObj();
    }
  }
  if (mousePressed == true){
    mouseFn();
  }
}

void sendMovement(SSource movedSource) {
  OscMessage myMessage = new OscMessage("/wfs");

  myMessage.add(movedSource.sourceNum); 
  myMessage.add(new float[] {
    (movedSource.xPos - originX) * pixScale,(originY - movedSource.yPos) * pixScale,0.0  }
  ); 
  myMessage.add(movedSource.sourceType);
  myMessage.add(movedSource.sourceGain);
  oscP5.send(myMessage, myRemoteLocation); 
}

void mouseFn() {
  if (sourceGrabbed == -1){
    for (int i=0; i < maxSources;i++){
      if (mySources[i].alive == true){
        if(mouseX > mySources[i].xPos - 5 && mouseX < mySources[i].xPos + 5 && mouseY > mySources[i].yPos - 5 && mouseY < mySources[i].yPos + 5) {
          sourceGrabbed = i;     
          nextOut = i;
          println("Moving " + i + "...");
          break;
        }
      }
    }
  }
  else{
    mySources[sourceGrabbed].move(mouseX, mouseY);
  }
  if (sourceGrabbed == -1){    
    if(bMakeSource.hovering()) {
      bMakeSource.pressing();
    }
    if(bRemoveSource.hovering()) {
      bRemoveSource.pressing();
    }
    if(bChangeSource.hovering()) {
      bChangeSource.pressing();
    }
    for(int i=0; i < numPresets; i++)
      if (preset[i].hovering())
        preset[i].pressing();
  }
}

void mouseReleased(){


  if (sourceGrabbed >= 0){
    println("moved to: " + (mySources[sourceGrabbed].xPos - originX) * pixScale + ", " + ((originY - mySources[sourceGrabbed].yPos) * pixScale + 1.0));
    sourceGrabbed = -1;
  }
  if (bMakeSource.mPressed == true){
    addSource();
  }
  if (bRemoveSource.mPressed == true){
    removeSource();
  }
  for(int i=0;i<8;i++){
    if (preset[i].mPressed == true){
 //     for(int j=0; j < maxSources; j++)
  //      mySources[j].goAway();
      preset(i);  
    }
  }
  for(int i=8;i<numPresets;i++){
    if (preset[i].mPressed == true){
      preset(i);  
    }
  }


  if (bChangeSource.mPressed == true){
    mySources[nextOut].changeType((mySources[nextOut].sourceType + 1) % 2);
  }
  bChangeSource.releasing();
  bMakeSource.releasing();  
  bRemoveSource.releasing();
  for (int i=0; i < numPresets; i++)
    preset[i].releasing();

}

void addSource(){
  for (int i=0;i<maxSources;i++){
    if(mySources[i].alive == true){
      continue;
    }
    else{
      mySources[i].comeBack(60 + 10 * i,60);
      nextOut = i;
      break;
    }
  }
}

void removeSource(){
  mySources[nextOut].goAway();
  for (int i=maxSources-1;i>=0;i--)
    if (mySources[i].alive == true){
      nextOut = i;
      break;
    }

}  

void preset(int num){

  switch(num){
    case(0):
    //surround sound
    println("Surround plane sources.");
    for (int i=0; i < 10; i++){
      mySources[i].sourceGain = 0.4;
      mySources[i].comeBack(48 + 54 * i, 500);
      mySources[i].changeType(1);
    }
  for(int j=10; j < maxSources; j++)
  mySources[j].goAway();
    break;
    case(1):
    //room effect
    println("Room reflection effect.");
    for (int i=0; i < 10; i++){
//      mySources[i].comeBack(0,0);
      mySources[i].sourceGain = 0.5;
      mySources[i].changeType(0);
    }
    mySources[0].comeBack(300,700);
    mySources[1].comeBack(320,540);
    mySources[2].comeBack(210,550);
    mySources[3].comeBack(390,560);
    mySources[4].comeBack(310,320);
    //mySources[5].move(260,470);
    mySources[5].comeBack(390,240);
    mySources[6].comeBack(470,150);

//    mySources[6].move(530,430);
    mySources[7].comeBack(100,110);
    mySources[8].comeBack(160,240);
    mySources[9].comeBack(70,300);
 //   mySources[10].move(390,240);
  //  mySources[11].move(470,150);
      for(int j=10; j < maxSources; j++)
  mySources[j].goAway();
    break;
    case(2):
    //stereo simulation
    println("Stero simulation");

    mySources[0].comeBack(260,670);
    mySources[1].comeBack(340,670);
    mySources[0].changeType(0);
    mySources[1].changeType(0);
      for(int j=2; j < maxSources; j++)
  mySources[j].goAway();
    break;
    case(3):
    println("Stero simulation. widened sweet spot.");
    for (int i=0; i < 2; i++){
      mySources[i].comeBack(0,0);
      mySources[i].sourceGain = 0.8;
      mySources[i].changeType(1);
    }
    mySources[0].move(260,670);
    mySources[1].move(340,670);
      for(int j=2; j < maxSources; j++)
  mySources[j].goAway();
    break;
    case(4):
    println("cocktail party. together");
    for (int i=0; i < 4; i++){
      mySources[i].sourceGain = 1.0;
      mySources[i].comeBack(280 + i * 5,500);
      mySources[i].changeType(0);
    }
      for(int j=4; j < maxSources; j++)
  mySources[j].goAway();
    break;
    case(5):
    println("cocktail party. apart");
    for (int i=0; i < 4; i++){
      mySources[i].sourceGain = 1.0;
      mySources[i].changeType(0);
    }
    mySources[0].comeBack(410,570);
    mySources[1].comeBack(220,640);
    mySources[2].comeBack(300,460);
    mySources[3].comeBack(320,220);
      for(int j=4; j < maxSources; j++)
  mySources[j].goAway();
    break;
    case(6):
    mySources[nextOut].sourceGain = 1.0;
    mySources[nextOut].movement = new CircleMove();
    mySources[nextOut].comeBack(0,0);
    break; 
    case(7):
    mySources[nextOut].sourceGain = 1.0;
    mySources[nextOut].movement = new EightMove();
    mySources[nextOut].comeBack(0,0);
    break; 
    case(8):
    mySources[nextOut].sourceGain = 1.0;
    mySources[nextOut].movement = new FarMove();
    mySources[nextOut].comeBack(0,0);
    break; 
    case(9):
    mySources[nextOut].sourceGain = 1.0;
    mySources[nextOut].movement = new AcrossMove();
    mySources[nextOut].comeBack(0,0);
    break; 
    case(10):
    mySources[nextOut].sourceGain = 1.0;
    mySources[nextOut].phase1 = mySources[nextOut].yPos;
    mySources[nextOut].phase2 = mySources[nextOut].xPos;
    mySources[nextOut].movement = new OutMove();
    break; 
  }
}


